from . import gtk
from . import treesel
from . import bencod
k=gtk.k

def go(path,window):
	with open(path,'rb') as f:
		d=f.read()
		a=bencod.decode(d)
		dialog = k.gtk_dialog_new_with_buttons (b"Info",window,
			gtk.GtkDialogFlags.GTK_DIALOG_DESTROY_WITH_PARENT | gtk.GtkDialogFlags.GTK_DIALOG_MODAL,
			b"_OK",gtk.GtkResponseType.GTK_RESPONSE_NONE,None)
		if k.gtk_window_is_maximized(window):
			k.gtk_window_maximize(dialog)
		else:
			width=gtk.c_int()
			height=gtk.c_int()
			k.gtk_window_get_default_size (window, gtk.byref(width), gtk.byref(height))
			k.gtk_window_set_default_size(dialog,width,height)
		k.g_signal_connect_data (dialog,b"response",k.gtk_window_destroy,None,None,0)
		box=k.gtk_dialog_get_content_area(dialog)
		#
		text = k.gtk_text_view_new ()
		k.gtk_text_view_set_editable(text, False)
		k.gtk_text_view_set_wrap_mode(text,gtk.GtkWrapMode.GTK_WRAP_WORD_CHAR)
		text_buffer = k.gtk_text_view_get_buffer (text)
		k.gtk_text_buffer_set_text (text_buffer,str(a).encode(),-1)
		#
		scrolled_window = k.gtk_scrolled_window_new ()
		k.gtk_scrolled_window_set_child (scrolled_window,text)
		k.gtk_box_append(box, scrolled_window)
		k.gtk_widget_set_hexpand(scrolled_window,True)
		k.gtk_widget_set_vexpand(scrolled_window,True)
		#
		k.gtk_widget_show (dialog)

def act(button,tree):
	treesel.text(go,tree,k.gtk_widget_get_root(button))



import os.path

from . import gtk
from . import layout
from . import torrent
from . import bencod
from . import sets
k=gtk.k

i=gtk.GtkTreeIter()
it=gtk.byref(i)

def ini(lst):
	global list
	list=lst
	k.gtk_list_store_append(list,it)
	global timer
	timer=k.g_timeout_add(5000,fresh,None)
	return list

def st(a):
	return str(a).encode()
def div_ratio(a,b):
	if not b:
		return "0"
	n=a/b
	return st(n)

def download_sz(mod,ir):
	item_text=gtk.c_char_p()
	gtk.gtk_tree_model_get (mod, ir, layout.COLUMNS.PATH, gtk.byref(item_text))
	val=item_text.value.decode()
	k.g_free(item_text)
	size=0
	try:
		with open(val,'rb') as f:
			dldir=k.gtk_entry_buffer_get_text(sets.fold_bf).decode()
			#join can concat bytes,but unicode decode otherwise filenotfound
			#	p.s. join cannot mix str and bytes
			d=f.read()
			cod=bencod.decode(d)
			infs=cod[0][b'info']
			fl=infs[b'name'].decode()
			if b'files' in infs:
				files=infs[b'files']
				dldir=os.path.join(dldir,fl)
				for x in files:
					p=x[b'path']
					nm=dldir
					for y in p:
						nm=os.path.join(nm,y.decode())
					size+=os.path.getsize(nm)
			else:
				size=os.path.getsize(os.path.join(dldir,fl))
	except Exception:
		pass
	return size

@gtk.CALLBACK0i
def fresh():
	up=0
	down=0
	i=gtk.GtkTreeIter()
	ir=gtk.byref(i)
	mod=layout.list
	b=k.gtk_tree_model_get_iter_first(mod,ir)
	j=0
	while b:
		tor=torrent.torrents[j]
		h=tor.h
		s=h.status()
		u=tor.u+s.total_payload_upload#if pause this will be 0, all_time_upload
		if torrent.checki(s):
			d=download_sz(mod,ir)
		else:
			d=s.total_done
		gtk.gtk_list_store_set3(mod,ir,layout.COLUMNS.UP,st(u),layout.COLUMNS.DOWN,st(d),layout.COLUMNS.RATIO,div_ratio(u,d))
		up+=u
		down+=d
		b=k.gtk_tree_model_iter_next(mod,ir)
		j+=1
	gtk.gtk_list_store_set3(list,it,layout.COLUMNS.UP,st(up),layout.COLUMNS.DOWN,st(down),layout.COLUMNS.RATIO,div_ratio(up,down))
	return True

def close():
	k.g_source_remove(timer)



from . import gtk
from . import ratio
from . import next
from . import log
from . import cons
k=gtk.k

fold_bf=k.gtk_entry_buffer_new(None,-1)

def labent(t,e):
	t=k.gtk_label_new (t)
	k.gtk_widget_set_hexpand(e,True)
	bx=k.gtk_box_new(gtk.GtkOrientation.GTK_ORIENTATION_HORIZONTAL,0)
	k.gtk_box_append(bx, t)
	k.gtk_box_append(bx, e)
	return bx

@gtk.CALLBACK3
def response(dialog,re,window):
	log.reset()#before ratio
	ratio.setint(window)
	cons.set()
	k.gtk_window_destroy(dialog)

@gtk.CALLBACK
def sets(window):
	dialog = k.gtk_dialog_new_with_buttons (b"Settings",window,
		gtk.GtkDialogFlags.GTK_DIALOG_DESTROY_WITH_PARENT | gtk.GtkDialogFlags.GTK_DIALOG_MODAL,
		b"_OK",gtk.GtkResponseType.GTK_RESPONSE_NONE,None)
	if k.gtk_window_is_maximized(window):
		k.gtk_window_maximize(dialog)
	else:
		width=gtk.c_int()
		height=gtk.c_int()
		k.gtk_window_get_default_size (window, gtk.byref(width), gtk.byref(height))
		k.gtk_window_set_default_size(dialog,width,height)
	k.g_signal_connect_data (dialog,b"response",response,window,None,0)
	#
	box=k.gtk_box_new(gtk.GtkOrientation.GTK_ORIENTATION_VERTICAL,0)
	e=k.gtk_entry_new_with_buffer(fold_bf)
	k.gtk_box_append(box, labent(b"Download Folder",e))
	k.gtk_box_append(box, ratio.ini())
	e=k.gtk_entry_new()
	next.sets(e)
	k.gtk_box_append(box, labent(b"Go to the next unfinished torrent",e))
	k.gtk_box_append(box, log.ini())
	k.gtk_box_append(box, labent(b"Max uploads (-1 infinite)",k.gtk_entry_new_with_buffer(cons.slot_bf)))
	k.gtk_box_append(box, labent(b"Max connections (-1 infinite)",k.gtk_entry_new_with_buffer(cons.con_bf)))
	#
	scw=k.gtk_scrolled_window_new()
	k.gtk_scrolled_window_set_child(scw,box)
	k.gtk_widget_set_vexpand(scw,True)
	bx=k.gtk_dialog_get_content_area(dialog)
	k.gtk_box_append(bx,scw)
	k.gtk_widget_show (dialog)
